==> klayout-0.14.tar.gz: klayout-0.14/src/layRedrawThread.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef HDR_layRedrawThread
#define HDR_layRedrawThread

#include <vector>
#include <set>

#include <QtCore/QThread>
#include <QtCore/QMutex>
#include <QtCore/QWaitCondition>

#include "dbBox.h"
#include "dbTrans.h"
#include "dbLayout.h"
#include "layRenderer.h"
#include "layLayoutView.h"
#include "layRedrawThreadCanvas.h"
#include "layBitmap.h"
#include "tlTimer.h"
#include "tlObserver.h"

namespace lay {

class Viewport;

//  update (snapshot) interval in ms
const int update_interval = 500;

//  some helpful constants
const int special_bitmaps_before = 2;
const int special_bitmaps_after = 1;
const int special_queue_entries = 2;
const int draw_boxes_queue_entry = -1;
const int draw_custom_queue_entry = -2;
const int bitmaps_per_layer = 8;

struct RedrawLayerInfo
{
  RedrawLayerInfo (const lay::LayerProperties &lp);

  bool visible;
  unsigned int layer_index;
  int cellview_index;
  std::vector<db::CplxTrans> trans;
  lay::HierarchyLevelSelection hier_levels;
  std::set <db::PropertiesRepository::properties_id_type> prop_sel;
  bool inverse_prop_sel;
};

class CellVariantCacheCompare
{
public:
  bool operator () (const std::pair<db::CplxTrans, db::Layout::cell_index_type> &a,
                    const std::pair<db::CplxTrans, db::Layout::cell_index_type> &b) const
  {
    if (a.second != b.second) {
      return a.second < b.second;
    }
    db::TransFuzzyLessF<db::CplxTrans> less;
    return less (a.first, b.first);
  }
};

class RedrawThread 
  : public QThread,
    public tl::Observer
{
public:
  typedef std::map <std::pair <db::CellGraph::cell_index_type, unsigned long>, int> MicroInstanceCache;

  RedrawThread (lay::RedrawThreadCanvas *canvas, lay::LayoutView *view);
  virtual ~RedrawThread ();

  void start (bool clear, const std::set<int> &layers, const lay::Viewport &vp, bool synchroneous);

  virtual void run ();
  void stop ();

  //  reimplementation of tl::Observer
  void signal_observed ();

private:
  QMutex m_main_lock;
  QMutex m_restart_lock;
  QWaitCondition m_restart_cond;
  QMutex m_initial_wait_lock;
  QWaitCondition m_initial_wait_cond;
  bool m_initial_update;
  const db::Layout *mp_layout;
  std::set <std::pair <db::CplxTrans, db::Layout::cell_index_type>, lay::CellVariantCacheCompare> *mp_cell_var_cache;
  unsigned int m_cache_hits, m_cache_misses;
  int m_cv_index;
  lay::RedrawThreadCanvas *mp_canvas;
  lay::LayoutView *mp_view;
  bool m_start_recursion_sentinel;
  bool m_request_stop;
  bool m_request_terminate;
  bool m_redraw;
  unsigned int m_test_count;
  int m_from_level, m_to_level;
  int m_from_level_default, m_to_level_default;
  bool m_drawing_context;
  size_t m_nlayers;
  bool m_box_text_transform;
  unsigned int m_box_font;
  int m_min_size_for_label;
  unsigned int m_text_font;
  bool m_text_visible;
  bool m_text_lazy_rendering;
  bool m_show_properties;
  bool m_apply_text_trans;
  double m_default_text_size;
  bool m_drop_small_cells;
  unsigned int m_drop_small_cells_value;
  lay::LayoutView::drop_small_cells_cond_type m_drop_small_cells_cond;
  std::vector <std::set <lay::LayoutView::cell_index_type> > m_hidden_cells;
  std::vector <RedrawLayerInfo> m_properties;
  std::set <std::pair <db::CplxTrans, int> > m_box_variants;
  std::vector <lay::CellView> m_cellviews;

  unsigned int m_layer;
  int m_level;
  MicroInstanceCache m_mi_cache;
  db::CplxTrans m_trans;
  const std::set<db::PropertiesRepository::properties_id_type> *mp_prop_sel;
  bool m_inv_prop_sel;
  db::Box m_vp;
  db::CellGraph::cell_index_type m_ci;
  lay::Renderer m_r;
  lay::Bitmap m_bitmaps [8];
  tl::Clock m_clock;
  std::vector<std::pair<unsigned int, lay::Bitmap *> > m_buffers;
  int m_width, m_height;
  db::DCplxTrans m_vp_trans;

  std::vector <int> m_draw_queue;
  std::set <int> m_already_drawn;

  bool m_synchroneous;

  void draw_layer ();
  void draw_boxes ();
  void draw_cell (const db::Box &box, const std::string &txt);
  void test_snapshot ();
  void transfer ();
  void iterate_variants (void (RedrawThread::*what) ());
  void iterate_variants_rec (void (RedrawThread::*what) ());
  bool cell_var_cached ();
  bool drop_cell (const db::Cell &cell);
};

}

#endif

==> klayout-0.15.tar.gz: klayout-0.15/src/layRedrawThread.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef HDR_layRedrawThread
#define HDR_layRedrawThread

#include <vector>
#include <set>

#include <QtCore/QThread>
#include <QtCore/QMutex>
#include <QtCore/QWaitCondition>

#include "dbBox.h"
#include "dbTrans.h"
#include "dbLayout.h"
#include "layRenderer.h"
#include "layLayoutView.h"
#include "layRedrawThreadCanvas.h"
#include "layBitmap.h"
#include "tlTimer.h"
#include "tlObserver.h"

namespace lay {

class Viewport;

//  update (snapshot) interval in ms
const int update_interval = 500;

//  some helpful constants
const int special_bitmaps_before = 2;
const int special_bitmaps_after = 1;
const int special_queue_entries = 2;
const int draw_boxes_queue_entry = -1;
const int draw_custom_queue_entry = -2;
const int bitmaps_per_layer = 8;

struct RedrawLayerInfo
{
  RedrawLayerInfo (const lay::LayerProperties &lp);

  bool visible;
  unsigned int layer_index;
  int cellview_index;
  std::vector<db::CplxTrans> trans;
  lay::HierarchyLevelSelection hier_levels;
  std::set <db::PropertiesRepository::properties_id_type> prop_sel;
  bool inverse_prop_sel;
};

class CellVariantCacheCompare
{
public:
  bool operator () (const std::pair<db::CplxTrans, db::Layout::cell_index_type> &a,
                    const std::pair<db::CplxTrans, db::Layout::cell_index_type> &b) const
  {
    if (a.second != b.second) {
      return a.second < b.second;
    }
    db::TransFuzzyLessF<db::CplxTrans> less;
    return less (a.first, b.first);
  }
};

class RedrawThread 
  : public QThread,
    public tl::Observer
{
public:
  typedef std::map <std::pair <db::CellGraph::cell_index_type, unsigned long>, int> MicroInstanceCache;

  RedrawThread (lay::RedrawThreadCanvas *canvas, lay::LayoutView *view);
  virtual ~RedrawThread ();

  void start (bool clear, const std::set<int> &layers, const lay::Viewport &vp, bool synchroneous);

  virtual void run ();
  void stop ();

  //  reimplementation of tl::Observer
  void signal_observed ();

private:
  QMutex m_main_lock;
  QMutex m_restart_lock;
  QWaitCondition m_restart_cond;
  QMutex m_initial_wait_lock;
  QWaitCondition m_initial_wait_cond;
  bool m_initial_update;
  const db::Layout *mp_layout;
  std::set <std::pair <db::CplxTrans, db::Layout::cell_index_type>, lay::CellVariantCacheCompare> *mp_cell_var_cache;
  unsigned int m_cache_hits, m_cache_misses;
  int m_cv_index;
  lay::RedrawThreadCanvas *mp_canvas;
  lay::LayoutView *mp_view;
  bool m_start_recursion_sentinel;
  bool m_request_stop;
  bool m_request_terminate;
  bool m_redraw;
  unsigned int m_test_count;
  int m_from_level, m_to_level;
  int m_from_level_default, m_to_level_default;
  bool m_drawing_context;
  size_t m_nlayers;
  bool m_box_text_transform;
  unsigned int m_box_font;
  int m_min_size_for_label;
  unsigned int m_text_font;
  bool m_text_visible;
  bool m_text_lazy_rendering;
  bool m_show_properties;
  bool m_apply_text_trans;
  double m_default_text_size;
  bool m_drop_small_cells;
  unsigned int m_drop_small_cells_value;
  lay::LayoutView::drop_small_cells_cond_type m_drop_small_cells_cond;
  std::vector <std::set <lay::LayoutView::cell_index_type> > m_hidden_cells;
  std::vector <RedrawLayerInfo> m_properties;
  std::set <std::pair <db::CplxTrans, int> > m_box_variants;
  std::vector <lay::CellView> m_cellviews;

  unsigned int m_layer;
  int m_level;
  MicroInstanceCache m_mi_cache;
  db::CplxTrans m_trans;
  const std::set<db::PropertiesRepository::properties_id_type> *mp_prop_sel;
  bool m_inv_prop_sel;
  db::Box m_vp;
  db::CellGraph::cell_index_type m_ci;
  db::PropertiesRepository::properties_id_type m_prop_id;
  lay::Renderer m_r;
  lay::Bitmap m_bitmaps [8];
  tl::Clock m_clock;
  std::vector<std::pair<unsigned int, lay::Bitmap *> > m_buffers;
  int m_width, m_height;
  db::DCplxTrans m_vp_trans;

  std::vector <int> m_draw_queue;
  std::set <int> m_already_drawn;

  bool m_synchroneous;

  void draw_layer ();
  void draw_boxes ();
  void draw_cell (const db::Box &box, const std::string &txt, db::PropertiesRepository::properties_id_type prop_id);
  void test_snapshot ();
  void transfer ();
  void iterate_variants (void (RedrawThread::*what) ());
  void iterate_variants_rec (void (RedrawThread::*what) ());
  bool cell_var_cached ();
  bool drop_cell (const db::Cell &cell);
};

}

#endif
